/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.finanzen;

import java.util.Date;

import org.threeten.bp.DateTimeUtils;
import org.threeten.bp.Instant;
import org.threeten.bp.LocalDate;
import org.threeten.bp.ZoneId;

/**
 * Ein Wertpapier-Wert Bean
 *
 * @author Frank Mnster
 */
public class HistoryValueBean implements Comparable<HistoryValueBean> {

    /**
     * Ein eindeutiger Identifikations String. Zum Beispiel das Yahoo-Symbol oder der Wertpapier Name
     */
    private String idString;

    /**
     * Das Datum
     */
    private Date date;

    /**
     * Der Er&ouml;ffnugskurs
     */
    private double open = Double.NaN;

    /**
     * Der Tagesh&ouml;chststand
     */
    private double high = Double.NaN;

    /**
     * Der Tagesniedrigsstand
     */
    private double low = Double.NaN;

    /**
     * Der Schlusskurs
     */
    private double close = Double.NaN;

    /**
     * Das Umsatzvolumen
     */
    private double volume;

    /**
     * der angepasste Schusskurs
     */
    private double adjClose = Double.NaN;

    /**
     * Standartkonstruktor
     */
    public HistoryValueBean() {
        // absichtlich leer;
    }

    /**
     * Liefert den eindeutigen Namen
     *
     * @return symbol der eindeutigen Namen
     */
    public String getIdString() {
        return this.idString;
    }

    /**
     * Setzt den eindeutigen Namen
     *
     * @param idString
     *            der eindeutigen Namen
     */
    public void setIdString(final String idString) {
        this.idString = idString;
    }

    /**
     * Liefert das Datum
     *
     * @return date das datum
     */
    public Date getDate() {
        return this.date;
    }

    /**
     * Setzt das Datum
     *
     * @param date
     *            das Datum
     */
    public void setDate(final Date date) {
        this.date = date;
    }

    /**
     * Setzt das Datum
     *
     * @param localDate
     *            das Datum als Java 8 LocalDate
     */
    public void setDate(final LocalDate localDate) {
        final Instant instant = localDate.atStartOfDay(ZoneId.systemDefault()).toInstant();
        this.date = DateTimeUtils.toDate(instant);
    }

    /**
     * Liefert den Er&ouml;ffnugskurs
     *
     * @return open Der Er&ouml;ffnugskurs
     */
    public double getOpen() {
        return this.open;
    }

    /**
     * Setzt den Er&ouml;ffnugskurs
     *
     * @param open
     *            Der Er&ouml;ffnugskurs
     */
    public void setOpen(final double open) {
        this.open = open;
    }

    /**
     * Liefert den Tagesh&ouml;chststand
     *
     * @return high der Tagesh&ouml;chststand
     */
    public double getHigh() {
        return this.high;
    }

    /**
     * Setzt den Tagesh&ouml;chststand
     *
     * @param high
     *            der Tagesh&ouml;chststand
     */
    public void setHigh(final double high) {
        this.high = high;
    }

    /**
     * Liefert den Tagesniedrigsstand
     *
     * @return low der Tagesniedrigsstand
     */
    public double getLow() {
        return this.low;
    }

    /**
     * Setzte den Tagesniedrigsstand
     *
     * @param low
     *            der Tagesniedrigsstand
     */
    public void setLow(final double low) {
        this.low = low;
    }

    /**
     * Liefert den Schlusskurs
     *
     * @return close der Schlusskurs
     */
    public double getClose() {
        return this.close;
    }

    /**
     * Setzt den Schlusskurs
     *
     * @param close
     *            der Schlusskurs
     */
    public void setClose(final double close) {
        this.close = close;
    }

    /**
     * Liefert das Umsatzvolumen
     *
     * @return volume das Umsatzvolumen
     */
    public double getVolume() {
        return this.volume;
    }

    /**
     * Setzt das Umsatzvolumen
     *
     * @param volume
     *            das Umsatzvolumen
     */
    public void setVolume(final double volume) {
        this.volume = volume;
    }

    /**
     * Liefert den angepasste Schlusskurs
     *
     * @return adjClose der angepasste Schlusskurs
     */
    public double getAdjClose() {
        return this.adjClose;
    }

    /**
     * Setzt den angepasste Schlusskurs
     *
     * @param adjClose
     *            der angepasste Schlusskurs
     */
    public void setAdjClose(final double adjClose) {
        this.adjClose = adjClose;
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    @Override
    public int compareTo(final HistoryValueBean historyValueBean) {
        int compareTo;
        if (historyValueBean.getDate() == null && this.getDate() == null) {
            compareTo = 0;
        } else if (this.getDate() == null) {
            compareTo = 1;
        } else if (historyValueBean.getDate() == null) {
            compareTo = -1;
        } else {
            compareTo = this.getDate().compareTo(historyValueBean.getDate());
        }
        return compareTo;
    }

    /**
     * Nice String for printout
     */
    @Override
    public String toString() {
        final StringBuilder result = new StringBuilder(this.idString);
        result.append(this.getDate().toString());
        return result.toString();
    }
}
