//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 


package de.frankmuenster.jameica.depot.finanzen.quandl;

import org.apache.commons.lang.StringUtils;

import com.jimmoores.quandl.QuandlSession;
import com.jimmoores.quandl.RetryPolicy;
import com.jimmoores.quandl.SessionOptions;

/**
 * @author Frank Mnster
 *
 */
public final class MyQuandlSession {
	
	/**
	 * Unsere Instanz
	 */
	private static MyQuandlSession instance;

	/**
	 * Die zugrundeliegende Quandl4J session
	 */
	private transient QuandlSession session;
	
	/**
	 * Der API-KEY
	 */
	private transient String authToken;

	/**
	 * privater Standardkonstruktor
	 * 
	 * @param authToken
	 *            Der API-KEY, kann NULL sein.
	 */
	private MyQuandlSession(final String authToken) {

		SessionOptions options;
		if (StringUtils.isEmpty(authToken)) {
			options = SessionOptions.Builder.withoutAuthToken().withRetryPolicy(RetryPolicy.createNoRetryPolicy())
					.build();
			session = QuandlSession.create(options);
		} else {
			options = SessionOptions.Builder.withAuthToken(authToken).withRetryPolicy(RetryPolicy.createNoRetryPolicy())
					.build();
			session = QuandlSession.create(options);
		}
	}
	
	/**
	 * Liefert die QuandlSearch Instanz.
	 * 
	 * @param authToken
	 *            Der API-KEY, kann NULL sein.
	 * 
	 * @return die QuandlSearch Instanz.
	 */
	public static QuandlSession getSession(final String authToken) {
		synchronized (QuandlSearch.class) {
			if (instance == null) {
				instance = new MyQuandlSession(authToken);
			} else if (!authToken.equals(instance.authToken)) {
				instance = new MyQuandlSession(authToken);
			}
			instance.authToken = authToken;
		}
		return instance.session;
	}
}
