//
// Copyright 2016 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.finanzen.quandl;

import java.util.ArrayList;
import java.util.List;

import com.jimmoores.quandl.MetaDataResult;
import com.jimmoores.quandl.QuandlSession;
import com.jimmoores.quandl.SearchRequest;
import com.jimmoores.quandl.SearchResult;
import com.jimmoores.quandl.util.QuandlRequestFailedException;

import de.frankmuenster.jameica.depot.finanzen.SearchWertpapier;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public final class QuandlSearch implements SearchWertpapier<QuandlStockInfoBean> {

	/**
	 * Der Quandl-Name
	 */
	private static final String QUNADL_NAME = "name"; //$NON-NLS-1$

	/**
	 * Text fr boolean false
	 */
	private static final String FALSE_TEXT = "false"; //$NON-NLS-1$

	/**
	 * Quandl Attribute fr Frequency-Typ
	 */
	private static final String QUANDL_FREQUENCY_ID = "frequency"; //$NON-NLS-1$ //NOPMD

	/**
	 * Quandl Attribute fr Serientyp
	 */
	private static final String QUANDL_TYPE_ID = "type"; //$NON-NLS-1$

	/**
	 * Quandl Attribute fr Premiumseries
	 */
	private static final String QUANDL_PREMIUM_ID = "premium"; //$NON-NLS-1$

	/**
	 * Quandl Wert fr tgliche Series
	 */
	private static final String QUNADL_FREQUENCY_DAILY = "daily"; //$NON-NLS-1$ //NOPMD

	/**
	 * Quandl Wert fr Zeitreihen
	 */
	private static final String QUNADL_TYPE_TIME_SERIES = "Time Series"; //$NON-NLS-1$ //NOPMD

	/**
	 * Unsere Instanz
	 */
	private static QuandlSearch instance;

	/**
	 * Die zugrundeliegende Quandl4J session
	 */
	private transient QuandlSession session;

	/**
	 * privater Standardkonstruktor
	 *
	 */
	private QuandlSearch() {
		// absichtlich leer
	}

	/**
	 * Liefert die QuandlSearch Instanz.
	 *
	 * @param authToken
	 *            Der API-KEY, kann NULL sein.
	 *
	 * @return die QuandlSearch Instanz.
	 */
	public static SearchWertpapier<QuandlStockInfoBean> getIntance(final String authToken) {
		synchronized (QuandlSearch.class) {
			if (QuandlSearch.instance == null) {
				QuandlSearch.instance = new QuandlSearch();
			}
		}
		QuandlSearch.instance.session = MyQuandlSession.getSession(authToken);
		return QuandlSearch.instance;
	}

	/**
	 * @see de.frankmuenster.jameica.depot.finanzen.SearchWertpapier#search(java.lang.String)
	 */
	@Override
	public List<QuandlStockInfoBean> search(final String name) {

		final List<QuandlStockInfoBean> list = new ArrayList<>();
		final SearchRequest sRequest = new SearchRequest.Builder().withQuery(name).withMaxPerPage(100).build();

		try {

			final SearchResult searchResult = this.session.search(sRequest);
			final List<MetaDataResult> resultList = new ArrayList<>();
			for (final MetaDataResult document : searchResult.getMetaDataResultList()) {

				try {
					// Nur freie Datenbank und Zeitreihen
					if (QuandlSearch.FALSE_TEXT.equals(document.getString(QuandlSearch.QUANDL_PREMIUM_ID))
							&& QuandlSearch.QUNADL_TYPE_TIME_SERIES
									.equals(document.getString(QuandlSearch.QUANDL_TYPE_ID))
							&& QuandlSearch.QUNADL_FREQUENCY_DAILY
									.equals(document.getString(QuandlSearch.QUANDL_FREQUENCY_ID))) {
						resultList.add(document);

						final QuandlStockInfoBean bean = new QuandlStockInfoBean(); // NOPMD
						bean.setCode(document.getQuandlCode());
						bean.setExchange(document.getString("database_code")); //$NON-NLS-1$
						bean.setName(document.getString(QuandlSearch.QUNADL_NAME));
						bean.setDescription(document.getString("description")); //$NON-NLS-1$
						bean.setRefeshedAt(document.getOffsetDateTime("refreshed-at")); //$NON-NLS-1$
						bean.setNewestAvailable(document.getLocalDate("newest-available-date")); //$NON-NLS-1$
						bean.setOldestAvailable(document.getLocalDate("oldest-available-date")); //$NON-NLS-1$

						list.add(bean);
					}

				} catch (RuntimeException e) { // NOPMD
					// Loggen und einfach weiter machen
					Logger.error(
							"Name: " + document.getString(QuandlSearch.QUNADL_NAME) + ". " + e.getLocalizedMessage()); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			Logger.debug("Usable codes: " + list.size() + " out of " + searchResult.getTotalDocuments()); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (QuandlRequestFailedException e) {
			// TODO: Fehler ermitteln (zu viele Requests...)
			Logger.info("Search request fails");
		}

		return list;
	}
}
