//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.StockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.quandl.QuandlStockInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooStockInfoBean;
import de.frankmuenster.jameica.depot.gui.dialog.SearchDialog;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart.SearchResultSelectListener;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.FfmSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.QuandlSearchPart;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.YahooSymbolSearchPart;
import de.frankmuenster.jameica.depot.gui.views.DepotView;
import de.frankmuenster.jameica.depot.gui.views.WertpapierDetailsView;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Kontrolleur fr die Wertpapier Bearbeiten Ansicht.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.ExcessiveImports" })
public class EditWertpapierController extends AbstractControl implements SearchResultSelectListener<StockInfoBean> {

	/**
	 *
	 */
	private static final String FFM_INSTRUMENT = "ffmInstrument"; //$NON-NLS-1$

	/**
	 * Das Suchfeldpart fr Yahoo
	 */
	private transient YahooSymbolSearchPart yahooSearchPart;

	/**
	 * Das Yahoo-Symbol des Wertpapieres
	 */
	private transient TextInput symbol;

	/**
	 * Der FFM Instrument Name des Wertpapieres
	 */
	private transient TextInput ffmInstrument;

	/**
	 * Der Such
	 */
	// private transient QuandlSearchPart quandlSearchPart;

	/**
	 * Der Quandl Code des Wertpapieres
	 */
	private transient TextInput quandlCode;

	/**
	 * Der Wertpapiername
	 */
	private transient TextInput name;

	/**
	 * Die W&auml;hrung des Kauf-/Verkaufpreis.
	 */
	private transient CurrencyInput stockCur;

	/**
	 * Kommentar
	 */
	private transient TextAreaInput comment;

	/**
	 * Das zuge&ouml;rigige Wertpapier-Objekt
	 */
	private transient Wertpapier wertpapier;

	/**
	 * Die W&auml;hrung des Wertpapiers
	 */
	private transient String wPapierWaehrung;

	/**
	 * Der Speicher-Button
	 */
	private transient Button store;

	/**
	 * Der Abbrechen-Button
	 */
	private transient Button cancle;

	/**
	 * Ist es ein neues Wertpapier?
	 */
	private transient boolean isNeuesWertpapier;

	/**
	 * Der Quandl-Code Such-Button
	 */
	private transient Button quandlSearchBtn;

	/**
	 * Der FFM Such-Button
	 */
	private transient Button ffmSearchBtn;

	/**
	 * Standardkonstruktor
	 */
	public EditWertpapierController(final AbstractView view) {
		super(view);
		final Object currentObject = this.getCurrentObject();
		if (currentObject instanceof Wertpapier) {
			Logger.debug("Bearbeite Wertpapier..."); //$NON-NLS-1$
			this.wertpapier = (Wertpapier) currentObject;
			try {
				this.wPapierWaehrung = ((Wertpapier) currentObject).getCurrency();
			} catch (RemoteException e) {
				Logger.warn("Whrung konnte nicht gelesen werden..." + //$NON-NLS-1$
						e.getMessage());
				// this.wPapierWaehrung = Settings.getDepotWaehrung();
			}
			this.isNeuesWertpapier = false;
		} else {
			this.isNeuesWertpapier = true;
		}
	}

	/**
	 * @return yahooSearchPart
	 */
	public YahooSymbolSearchPart getYahooSearchPart() {
		if (this.yahooSearchPart == null) {
			this.yahooSearchPart = new YahooSymbolSearchPart(this);
		}
		return this.yahooSearchPart;
	}

	/**
	 * @return symbol
	 * @throws RemoteException
	 */
	public TextInput getSymbol() throws RemoteException {
		if (this.symbol == null) {
			this.symbol = new TextInput(this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getSymbol());
		}
		return this.symbol;
	}

	/**
	 * @return name
	 * @throws RemoteException
	 */
	public TextInput getName() throws RemoteException {
		if (this.name == null) {
			this.name = new TextInput(this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getName());
		}
		return this.name;
	}

	/**
	 * Whrung
	 *
	 * @return Whrungsauswahlfeld
	 */
	public CurrencyInput getStockCurrency() {
		if (this.stockCur == null) {
			this.stockCur = new CurrencyInput(this.wPapierWaehrung);
			this.stockCur.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.warn(event.toString());
					if (event.type == SWT.Selection) {
						EditWertpapierController.this.wPapierWaehrung = (String) EditWertpapierController.this.stockCur
								.getValue();
					}
				}
			});
		}
		return this.stockCur;
	}

	/**
	 * @return ffmInstrument
	 * @throws RemoteException
	 */
	public TextInput getFfmInstrument() throws RemoteException {
		if (this.ffmInstrument == null) {
			this.ffmInstrument = new TextInput(StringUtils.EMPTY);

			if (this.wertpapier == null) {
				this.ffmInstrument.setData(EditWertpapierController.FFM_INSTRUMENT, StringUtils.EMPTY);
			} else {
				this.ffmInstrument.setValue(this.wertpapier.getFfmName());
				this.ffmInstrument.setData(EditWertpapierController.FFM_INSTRUMENT, this.wertpapier.getFfmInstrument());
			}

		}
		return this.ffmInstrument;
	}

	/**
	 * @return quandlCode
	 * @throws RemoteException
	 */
	public TextInput getQuandlCode() throws RemoteException {
		if (this.quandlCode == null) {
			final String code = this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getQuandlCode();
			this.quandlCode = new TextInput(code);
		}
		return this.quandlCode;
	}

	/**
	 * Liefert den Ffm Suchen-Button.
	 *
	 * @return den Ffm Suchen-Button
	 */
	public Button getFfmSearchBtn() {
		if (this.ffmSearchBtn == null) {
			this.ffmSearchBtn = new Button("...", new Action() { //$NON-NLS-1$

				/**
				 * @see Action#handleAction(Object)
				 */
				@Override
				public void handleAction(final Object context) throws ApplicationException {
					// Open SearchDialog
					String name = StringUtils.EMPTY;
					try {
						name = EditWertpapierController.this.wertpapier == null ? StringUtils.EMPTY
								: EditWertpapierController.this.wertpapier.getName();
					} catch (RemoteException e) {
						// Nur Loggen
						Logger.debug("Fehler beim Lesen des FFM Wertpapiers. " + e.getLocalizedMessage()); //$NON-NLS-1$
					}

					final SearchDialog<FfmStockInfoBean, FfmSearchPart> searchDialog = new SearchDialog<FfmStockInfoBean, FfmSearchPart>(
							SearchDialog.POSITION_CENTER, name, FfmSearchPart.class);
					try {
						final Object result = searchDialog.open();

						if (result instanceof FfmStockInfoBean) {
							EditWertpapierController.this.getFfmInstrument()
									.setValue(((FfmStockInfoBean) result).getName());
							EditWertpapierController.this.getFfmInstrument().setData(
									EditWertpapierController.FFM_INSTRUMENT,
									((FfmStockInfoBean) result).getInstrumentId());
						}

					} catch (Exception e) { // NOPMD
						Logger.error(Settings.getI18n().tr("EditWertpapierController.dialogError"), //$NON-NLS-1$
								e);
						throw new ApplicationException(Settings.getI18n().tr("EditWertpapierController.dialogError"), //$NON-NLS-1$
								e);
					}
				}
			});
		}
		return this.ffmSearchBtn;
	}

	/**
	 * Liefert den Quandl-Code Suchen-Button.
	 *
	 * @return den Quandl-Code Suchen-Button
	 */
	public Button getQuandlSearchBtn() {

		if (this.quandlSearchBtn == null && StringUtils.isNotEmpty(Settings.getQuandlApiKey())) {
			this.quandlSearchBtn = new Button("...", new Action() { //$NON-NLS-1$

				/**
				 * @see Action#handleAction(Object)
				 */
				@Override
				public void handleAction(final Object context) throws ApplicationException {
					// Open SearchDialog
					String code = StringUtils.EMPTY;
					try {
						code = EditWertpapierController.this.wertpapier == null ? StringUtils.EMPTY
								: EditWertpapierController.this.wertpapier.getQuandlCode();
					} catch (RemoteException e) {
						// Nur Loggen
						Logger.debug("Fehler beim Lesen des Quandl-Code des Wertpapiers. " + e.getLocalizedMessage()); //$NON-NLS-1$
					}

					final SearchDialog<QuandlStockInfoBean, QuandlSearchPart> searchDialog = new SearchDialog<QuandlStockInfoBean, QuandlSearchPart>(
							SearchDialog.POSITION_CENTER, code, QuandlSearchPart.class);
					try {
						final Object result = searchDialog.open();

						if (result instanceof QuandlStockInfoBean) {
							EditWertpapierController.this.getQuandlCode()
									.setValue(((QuandlStockInfoBean) result).getCode());
						}

					} catch (Exception e) { // NOPMD
						throw new ApplicationException(Settings.getI18n().tr("EditWertpapierController.dialogError"), //$NON-NLS-1$
								e);
					}
				}
			});
		}
		return this.quandlSearchBtn;
	}

	/**
	 * @return comment
	 * @throws RemoteException
	 */
	public TextAreaInput getComment() throws RemoteException {
		if (this.comment == null) {
			this.comment = new TextAreaInput(
					this.wertpapier == null ? StringUtils.EMPTY : this.wertpapier.getComment());
		}
		return this.comment;
	}

	/**
	 * Liefert den Speichern-Button
	 *
	 * @return den Speichern-Button
	 */
	public Button getStore() {
		if (this.store == null) {
			this.store = new Button(Settings.getI18n().tr("saveBtn"), new Action() { //$NON-NLS-1$

				/**
				 * @see Action#handleAction(Object context)
				 */
				@Override
				public void handleAction(final Object context) throws ApplicationException {
					try {
						// Wertpapier daten speichern
						EditWertpapierController.this.handleStore();

						// Wertpapier anzeigen
						GUI.startView(WertpapierDetailsView.class.getName(), EditWertpapierController.this.wertpapier);
					} catch (RemoteException e) {
						Logger.error(Settings.getI18n()
								.tr("EditWertpapierController.saveStockError" + e.getLocalizedMessage())); //$NON-NLS-1$
						throw new ApplicationException(Settings.getI18n().tr("EditWertpapierController.saveStockError"), //$NON-NLS-1$
								e);
					}
				}
			}, null, true, "save-icon.png"); //$NON-NLS-1$
			this.store.setEnabled(true);
		}
		return this.store;
	}

	/**
	 * Liefert den Abbrechen-Button
	 *
	 * @return den Abbrechen-Button
	 */
	public Button getCancel() {
		if (this.cancle == null) {
			this.cancle = new Button(Settings.getI18n().tr("cancelBtn"), //$NON-NLS-1$
					new Action() {

						/**
						 * @see Action#handleAction(Object context)
						 */
						@Override
						public void handleAction(final Object context) throws ApplicationException {
							if (EditWertpapierController.this.isNeuesWertpapier) {
								GUI.startView(DepotView.class, null);
							} else {
								GUI.startView(WertpapierDetailsView.class, EditWertpapierController.this.wertpapier);
							}
						}
					}, null, false, "cancle-icon.png"); //$NON-NLS-1$
		}
		return this.cancle;
	}

	/**
	 * @see de.frankmuenster.jameica.depot.gui.parts.YahooSymbolSearchPart.SearchResultSelectListener#searchResultSelected(de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooStockInfoBean)
	 */
	@Override
	public void searchResultSelected(final StockInfoBean stockInfoBean) {
		if (stockInfoBean instanceof YahooStockInfoBean) {
			this.handleYahooSelected((YahooStockInfoBean) stockInfoBean);
		} else {
			this.handleQuandlSelected((QuandlStockInfoBean) stockInfoBean);
		}
	}

	/**
	 * @param stockInfoBean
	 */
	private void handleQuandlSelected(final QuandlStockInfoBean qStockInfoBean) {
		this.quandlCode.setValue(qStockInfoBean.getCode());
	}

	private void handleYahooSelected(final YahooStockInfoBean yStockInfoBean) {
		Logger.debug("Yahoo Search result ausgewhlt..." + yStockInfoBean.toString()); //$NON-NLS-1$
		this.symbol.setValue(yStockInfoBean.getSymbol());
		this.name.setValue(yStockInfoBean.getName());
		// Whrung und aktuellen Kurs ermitteln
		try {
			final WertpapierInfoBean wPapierInfoBean = YahooFinanceAPI.getInstance()
					.getWertpapierInfoCSV(yStockInfoBean.getSymbol(), true);
			if (wPapierInfoBean != null) {
				// neuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
				this.wPapierWaehrung = wPapierInfoBean.getCurrency();
				this.stockCur.setValue(this.wPapierWaehrung);
			}
		} catch (FinanceException ye) {
			// Melden, das Kurs nicht gefunden wurde
			final StringBuffer msg = new StringBuffer();
			msg.append(Settings.getI18n().tr("EditWertpapierController.stockInfoNotFoundMsg", //$NON-NLS-1$
					(String) this.symbol.getValue()));
			Application.getMessagingFactory()
					.sendMessage(new StatusBarMessage(msg.toString(), StatusBarMessage.TYPE_INFO));
			this.wPapierWaehrung = Settings.getDepotWaehrung();
		}
	}

	private void handleStore() throws RemoteException, ApplicationException {
		final DBService service = Settings.getDBService();
		if (this.wertpapier == null) {
			// Es muss ein neues WP angelegt werden.
			this.wertpapier = (Wertpapier) service.createObject(Wertpapier.class, null);
		}
		this.wertpapier.setSymbol((String) this.getSymbol().getValue());
		this.wertpapier.setName((String) this.getName().getValue());
		this.wertpapier.setFfmName((String) this.getFfmInstrument().getValue());
		this.wertpapier
				.setFfmInstrument((String) this.getFfmInstrument().getData(EditWertpapierController.FFM_INSTRUMENT));
		this.wertpapier.setQuandlCode((String) this.getQuandlCode().getValue());
		this.wertpapier.setCurrency((String) this.getStockCurrency().getValue());
		this.wertpapier.setComment((String) this.getComment().getValue());
		// Aktuellen Kurs ermitteln
		try {
			final WertpapierInfoBean wPapierInfoBean = YahooFinanceAPI.getInstance()
					.getWertpapierInfoCSV(this.wertpapier.getSymbol(), false);
			if (wPapierInfoBean != null) {
				// isNeuesWertpapier.setExchange(wertpapierInfo.getStockExchange());
				this.wertpapier.setLastTradePriceOnly(wPapierInfoBean.getLastTradePriceOnly());
				this.wertpapier.setLastTradeDate(wPapierInfoBean.getLastTradeDate());
			}
		} catch (FinanceException ye) {
			// OK, dann halt nicht.
			Logger.info(Settings.getI18n().tr("EditWertpapierController.noFinancialInfoFound")); //$NON-NLS-1$
		}
		this.wertpapier.store();
	}
}
