/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.WertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmStockInfoBean;
import de.frankmuenster.jameica.depot.gui.action.OrderDetailAction;
import de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInputListener;
import de.frankmuenster.jameica.depot.gui.menu.OrderListeMenu;
import de.frankmuenster.jameica.depot.gui.parts.DiagrammAnzeigeAuswahlPart;
import de.frankmuenster.jameica.depot.gui.parts.OrderListePart;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierDetailsPart;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierPerformanzPart;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.kursgrafik.KursGrafik;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DateInput;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ContextMenu;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

//TODO: Grafikwerkzeuge als eigenstndiges Part mit eigenem Event-Handler.
//Ergebnisse sollten dann in einem Event argument and Listener verteilt werden.
/**
 * Komplexer Wertpapier Anzeige-Kontroller
 *
 * @author munstfr
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass", "PMD.TooManyFields" })
public class WertpapierDetailController extends AbstractControl
        implements WertpapierAktualisiertListener, OrderDeleteListener {

    /**
     * Das Wertpapier das wird kontrollieren wollen
     */
    private transient Wertpapier wertpapier;

    /**
     * Symbol-Eingabefeld
     */
    private transient TextInput symbol;

    /**
     * Name-Eingabefeld
     */
    private transient TextInput name;

    /**
     * Kommentar-Eingabefeld
     */
    private transient TextAreaInput comment;

    /**
     * Die Performanz-Anzeige
     */
    private transient WertpapierPerformanzPart performanzPart;

    /**
     * Datum-Eingabefeld
     */
    private transient DateInput datum;

    /**
     * Kurswert-Eingabefeld
     */
    private transient DecimalInput kurs;

    /**
     * W&auml;hrungs-Eingabefeld
     */
    private transient CurrencyInput waehrung;

    /**
     * OrderListe
     */
    private transient TablePart orderHistoryList;

    /**
     * Grafikwerkzeug - Zeitschhieber TODO: in externe Klasse
     */
    private transient ZeitauswahlSchieberInput zeitAuswahl;

    /**
     * Grafikwerkzeug - ZeitAuswahlButtons TODO: in externe Klasse
     */
    private transient PeriodChooserInput zeitAuswahlBttns;

    /**
     * Grafikwerkzeug - ZeigeDax TODO: in externe Klasse
     */
    private transient CheckboxInput showDax;

    /**
     * Grafikwerkzeug - Zeige als Kerzengrafik TODO: in externe Klasse
     */
    private transient CheckboxInput showCandle;

    /**
     * Container f&uuml;r Grafikwerzeuge TODO: Als vollwertige Klasse mit Event-Handler erweitern
     */
    private transient DiagrammAnzeigeAuswahlPart grafikWerkzeuge;

    /**
     * Kursgrafik
     */
    private transient KursGrafik grafik;

    /**
     * Details zum Wertpapier TODO: Check ob als vollwertige Klasse implementiert werden kann.
     */
    private transient WertpapierDetailsPart wpDetailsPart;

    /**
     * Historische Werte des Wertpapiers
     */
    private transient WertpapierHistory wertpapierHistory;

    /**
     * Historische DAX-Werte
     */
    private transient WertpapierHistory daxHistory;

    /**
     * Sind historische Werte verf&uuml;gbar
     */
    private transient boolean histDataAvailable;

    /**
     * Buttonm zum Speichern
     */
    private transient Button saveWertpapier;

    /**
     * @param view
     * @throws RemoteException
     */
    public WertpapierDetailController(final AbstractView view) throws RemoteException {
        super(view);
        final String symbol = this.getWertPapier().getSymbol();
        // final String name = this.getWertPapier().getName();
        this.loadData(symbol);
        this.loadDaxData();
        if (this.wertpapierHistory != null && this.daxHistory != null) {
            this.histDataAvailable = true;
        }
    }

    /**
     * Liefert das aktuelle Wertpapier.
     *
     * @return das WertPapier.
     */
    private Wertpapier getWertPapier() {
        if (this.wertpapier == null) {
            this.wertpapier = (Wertpapier) this.getCurrentObject();
        }
        return this.wertpapier;
    }

    /**
     * Liefert true, falls die Historiedaten geladen werden konnten.
     *
     * @return true wenn die Kurs- und Daxhistorie geladen werden konnte, false sonst.
     */
    public boolean isHistoDataAvailable() {
        return this.histDataAvailable;
    }

    /**
     * Returns the wertpapierDetails Part for the symbol name.
     *
     * @return part wertpapierDetails part
     * @throws RemoteException
     */
    public WertpapierDetailsPart getWertpapierDetails() throws RemoteException {
        if (this.wpDetailsPart == null) {
            this.wpDetailsPart = new WertpapierDetailsPart(this.wertpapier, this);
        }
        return this.wpDetailsPart;
    }

    /**
     * Returns the input field for the symbol name.
     *
     * @return input field.
     * @throws RemoteException
     */
    public Input getSymbol() throws RemoteException {
        if (this.symbol == null) { // "255" is the maximum length for this input
                                   // field.
            this.symbol = new TextInput(this.getWertPapier().getSymbol(), 255);
            this.symbol.setMandatory(true);
            this.symbol.setName(Settings.getI18n().tr("WertpapierDetailController.symbol")); //$NON-NLS-1$
        }
        return this.symbol;
    }

    /**
     * Returns the input field for the project name.
     *
     * @return input field.
     * @throws RemoteException
     */
    public Input getName() throws RemoteException {
        if (this.name == null) { // "255" is the maximum length for this input
                                 // field.
            this.name = new TextInput(this.getWertPapier().getName(), 255);
            this.name.setMandatory(true);
            this.name.setName(Settings.getI18n().tr("WertpapierDetailController.name")); //$NON-NLS-1$
        }
        return this.name;
    }

    /**
     * Liefert ein {@link DateInput} filed mit den LastTrade Feld der Aktie aus der Datenbank.
     *
     * @return DateInput field
     * @throws RemoteException
     */
    public DateInput getDatum() throws RemoteException {
        if (this.datum == null) {
            final DateFormat format = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
                    Locale.GERMAN);
            final Date lastTradeDate = this.getWertPapier().getLastTradeDate();
            this.datum = new DateInput(lastTradeDate, format);
            this.datum.setTitle(Settings.getI18n().tr("WertpapierDetailController.lastTradeDate")); //$NON-NLS-1$
            this.datum.setText(Settings.getI18n().tr("WertpapierDetailController.choseDate")); //$NON-NLS-1$
        }
        return this.datum;
    }

    /**
     * Liefert ein {@link CurrencyInput} field mit der W&auml;hrung der Aktie aus der Datenbank.
     *
     * @return DateInput field
     * @throws RemoteException
     */
    public CurrencyInput getWaehrung() throws RemoteException {
        if (this.waehrung == null) {
            this.waehrung = new CurrencyInput(this.getWertPapier().getCurrency());
        }
        return this.waehrung;
    }

    /**
     * Liefert ein {@link DecimalInput} mit dem Kurswert der Aktie aus der Datenbank.
     *
     * @return DecimalInput field.
     * @throws RemoteException
     */
    public DecimalInput getLetzterKursWert() throws RemoteException {
        if (this.kurs == null) {
            final DecimalFormat format = new DecimalFormat("###,###,##0.00"); //$NON-NLS-1$
            this.kurs = new DecimalInput(this.getWertPapier().getLastTradePriceOnly(), format);
        }
        return this.kurs;
    }

    /**
     * Liefert das OrderHistoryPart
     *
     * @return das OrderHistoryPart
     * @throws RemoteException
     */
    public Part getOrderHistoryList() throws RemoteException {
        if (this.orderHistoryList == null) {
            final DBIterator<?> orderHistory = this.getWertPapier().getOrderHistory();
            this.orderHistoryList = new OrderListePart(orderHistory, new OrderDetailAction());
            // Kontext Men hinzufgen
            final ContextMenu menu = new OrderListeMenu(this.wertpapier, this);
            this.orderHistoryList.setContextMenu(menu);
        }
        return this.orderHistoryList;
    }

    /**
     * Liefert das Kommentarfeld
     *
     * @return das Kommentarfeld
     * @throws RemoteException
     */
    public Input getComment() throws RemoteException {
        if (this.comment == null) {
            this.comment = new TextAreaInput(this.getWertPapier().getComment());
        }
        return this.comment;
    }

    /**
     * Liefert den Chart
     *
     * @return den Chart
     */
    public AbstractGrafik getGrafik() {
        if (this.grafik == null && this.wertpapierHistory != null && this.daxHistory != null) {
            this.grafik = new KursGrafik();
            this.grafik.setKursDaxDaten(this.wertpapierHistory, this.daxHistory);
            if (this.zeitAuswahl == null) {
                this.grafik.setFromDate(this.wertpapierHistory.getFromDate());
                this.grafik.setToDate(this.wertpapierHistory.getToDate());
            } else {
                this.grafik.setFromDate(this.zeitAuswahl.getLower());
                this.grafik.setToDate(this.zeitAuswahl.getUpper());
            }
            this.grafik.setShowAsCandleStick(false);
            this.grafik.setShowDax(false);
        }
        return this.grafik;
    }

    /**
     * Liefert das Zeitauswahl-Part
     *
     * @return das Zeitauswahl-Part
     */
    public Part getZeitauswahl() {
        if (this.zeitAuswahl == null) {
            this.zeitAuswahl = new ZeitauswahlSchieberInput();
            // Zeitachse setzten und das letzte Jahr auswhlen
            this.zeitAuswahl.setScaling(this.wertpapierHistory.getFromDate(), this.wertpapierHistory.getToDate(), 1,
                    10);
            this.zeitAuswahl.selectLastYear();
            // Platz fr den Kommentar schaffen
            this.zeitAuswahl.setComment(""); //$NON-NLS-1$
            this.zeitAuswahl.addListener(new ZeitAuswahlListener());
        }
        return this.zeitAuswahl;
    }

    /**
     * Liefert die Grafikwerkzeuge in einem Part
     *
     * @return DiagrammAnzeigeAuswahlPart (Grafikwerkzeuge)
     */
    public DiagrammAnzeigeAuswahlPart getGrafikWerkzeuge() {
        if (this.grafikWerkzeuge == null && this.wertpapierHistory != null) {
            this.grafikWerkzeuge = new DiagrammAnzeigeAuswahlPart();
            if (this.daxHistory == null) {
                this.grafikWerkzeuge.setDax(null);
            } else {
                this.grafikWerkzeuge.setDax(this.getShowDax());
            }
            this.grafikWerkzeuge.setPeriod((PeriodChooserInput) this.getZeitauswahlButtons());
            this.grafikWerkzeuge.setHighLow(this.getShowCandle());
        }
        return this.grafikWerkzeuge;
    }

    private Part getZeitauswahlButtons() {
        if (this.zeitAuswahlBttns == null) {
            this.zeitAuswahlBttns = new PeriodChooserInput();
            this.zeitAuswahlBttns.addListener(new ZeitAuswahlButtonListener());
        }
        return this.zeitAuswahlBttns;
    }

    private CheckboxInput getShowDax() {
        if (this.showDax == null) {
            this.showDax = new CheckboxInput(false);
            this.showDax.setName("DAX"); //$NON-NLS-1$
            this.showDax.setEnabled(true);
            this.showDax.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    Logger.warn("DAX:" + WertpapierDetailController.this.showDax.getValue()); //$NON-NLS-1$
                    if (event.type == SWT.Selection) {
                        WertpapierDetailController.this.grafik
                                .setShowDax((Boolean) WertpapierDetailController.this.showDax.getValue());
                        if ((Boolean) WertpapierDetailController.this.showDax.getValue()) {
                            WertpapierDetailController.this.showCandle.setValue(false);
                        }
                        WertpapierDetailController.this.grafik.reload();
                    }
                }
            });
        }
        return this.showDax;
    }

    private CheckboxInput getShowCandle() {
        if (this.showCandle == null) {
            this.showCandle = new CheckboxInput(false);
            this.showCandle.setName(Settings.getI18n().tr("WertpapierDetailController.candle")); //$NON-NLS-1$
            this.showCandle.addListener(new Listener() {

                /**
                 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event event)
                 */
                @Override
                public void handleEvent(final Event event) {
                    Logger.warn("LowHigh:" + WertpapierDetailController.this.showCandle.getValue()); //$NON-NLS-1$
                    if (event.type == SWT.Selection) {
                        WertpapierDetailController.this.grafik
                                .setShowAsCandleStick((Boolean) WertpapierDetailController.this.showCandle.getValue());
                        if ((Boolean) WertpapierDetailController.this.showCandle.getValue()) {
                            WertpapierDetailController.this.showDax.setValue(false);
                        }
                        WertpapierDetailController.this.grafik.reload();
                    }
                }
            });
        }
        return this.showCandle;
    }

    private void loadData(final String symbol) {
        if (StringUtils.isEmpty(symbol)) {
            this.wertpapierHistory = null; // NOPMD
        } else {
            try {
                final FfmStockInfoBean ffmInfoBean = new FfmStockInfoBean(this.getWertPapier().getFfmInstrument(),
                        this.getWertPapier().getFfmName(), null, null);
                this.wertpapierHistory = new WertpapierHistory(this.getWertPapier().getSymbol(),
                        this.getWertPapier().getQuandlCode(), ffmInfoBean);
            } catch (final FinanceException e) {
                this.wertpapierHistory = null; // NOPMD
            } catch (final RemoteException e) {
                this.wertpapierHistory = null; // NOPMD
            }
        }
    }

    private void loadDaxData() {
        try {
            final FfmStockInfoBean ffmInfoBean = new FfmStockInfoBean(null, "DAX 30", null, null); //$NON-NLS-1$
            this.daxHistory = new WertpapierHistory("^GDAXI", null, ffmInfoBean); //$NON-NLS-1$
        } catch (final FinanceException e) {
            this.daxHistory = null; // NOPMD
        }
    }

    /**
     * Private Listener Klasse, die auf den ZeitauswahlSchieber reagiert.
     *
     * @author munstfr
     */
    private class ZeitAuswahlListener implements ZeitauswahlSchieberInputListener {

        /**
         * Standardkonstruktor
         */
        public ZeitAuswahlListener() {
            // absichtlich leer
        }

        /**
         * @see ZeitauswahlSchieberInputListener#rangeChanged()
         */
        @Override
        public void rangeChanged() {
            Logger.warn(Settings.getI18n().tr("WertpapierDetailController.timeRangeChnaged")); //$NON-NLS-1$
            WertpapierDetailController.this.grafik.setFromDate(WertpapierDetailController.this.zeitAuswahl.getLower());
            WertpapierDetailController.this.grafik.setToDate(WertpapierDetailController.this.zeitAuswahl.getUpper());
            WertpapierDetailController.this.grafik.reload();
        }
    }

    /**
     * Private Listener Klasse, die auf den ZeitAuswahlButton reagiert.
     *
     * @author munstfr
     */
    private class ZeitAuswahlButtonListener implements Listener {

        /**
         * Standardkonstruktor
         */
        public ZeitAuswahlButtonListener() {
            // absichtlich leer
        }

        /**
         * @see ZeitAuswahlButtonListener#handleEvent(Event)
         */
        @Override
        public void handleEvent(final Event event) {
            if (event.type != SWT.FocusIn && event.type != SWT.FocusOut
                    && event.widget instanceof org.eclipse.swt.widgets.Button) {
                final int tage = (Integer) RadioInput.getValue(PeriodChooserInput.GROUP_ID);
                final int toDate = WertpapierDetailController.this.zeitAuswahl.getMaximun();
                WertpapierDetailController.this.zeitAuswahl.setUpper(toDate);
                WertpapierDetailController.this.zeitAuswahl.setLower(toDate - tage);
                WertpapierDetailController.this.grafik
                        .setFromDate(WertpapierDetailController.this.zeitAuswahl.getLower());
                WertpapierDetailController.this.grafik
                        .setToDate(WertpapierDetailController.this.zeitAuswahl.getUpper());
                WertpapierDetailController.this.grafik.reload();
            }
        }
    }

    /**
     * Liefert den Speichern-Button
     *
     * @return den Speichern-Button
     */
    public Button getSave() {
        if (this.saveWertpapier == null) {
            this.saveWertpapier = new Button(
                    Settings.getI18n().tr(Settings.getI18n().tr("WertpapierDetailController.save")), //$NON-NLS-1$
                    new Action() {

                        /**
                         * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
                         */
                        @Override
                        public void handleAction(final Object context) throws ApplicationException {
                            WertpapierDetailController.this.handleSaveWertpapier();
                        }
                    });
        }
        return this.saveWertpapier;
    }

    /**
     * Liefert das WertpapierPerformanzPart.
     *
     * @return das Performanz-Part
     */
    public WertpapierPerformanzPart getPerformanzPart() {
        if (this.performanzPart == null) {
            this.performanzPart = new WertpapierPerformanzPart(this.wertpapier);
        }
        return this.performanzPart;
    }

    private void handleSaveWertpapier() {
        try {
            if (!this.wertpapier.isInYahoo()) {
                this.wertpapier.setLastTradeDate((Date) this.getDatum().getValue());
                this.wertpapier.setLastTradePriceOnly((Double) this.getLetzterKursWert().getValue());
                this.wertpapier.setCurrency((String) this.getWaehrung().getValue());
            }
            this.wertpapier.setComment((String) this.getComment().getValue());
            this.wertpapier.store();
            Application.getMessagingFactory().sendMessage(
                    new StatusBarMessage(Settings.getI18n().tr("WertpapierDetailController.commentSaveSuccess"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_SUCCESS));
        } catch (RemoteException | ApplicationException e) {
            final String message = Settings.getI18n().tr("WertpapierDetailController.commentSaveError"); //$NON-NLS-1$
            Application.getMessagingFactory().sendMessage(new StatusBarMessage(message, StatusBarMessage.TYPE_ERROR));
            Logger.error(e.getMessage());
        }
        try {
            this.getWertpapierDetails().update();
        } catch (final RemoteException e) {
            final String message = Settings.getI18n().tr("WertpapierDetailController.updateViewFailed"); //$NON-NLS-1$
            Application.getMessagingFactory().sendMessage(new StatusBarMessage(message, StatusBarMessage.TYPE_ERROR));
        }
    }

    private void updateFelder() {
        // Wertpapierdeatail Felder aktualisieren
        if (this.wpDetailsPart != null) {
            this.wpDetailsPart.update();
        }
        if (this.performanzPart != null) {
            this.performanzPart.update();
        }
    }

    /**
     * @see WertpapierAktualisiertListener#notifyWertpapiereAktualisiert()
     */
    @Override
    public void notifyWertpapiereAktualisiert() {
        this.updateFelder();
    }

    /**
     * @see de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener#notifyOrderDeleted()
     */
    @Override
    public void notifyOrderDeleted() {
        this.updateFelder();
    }
}
