/*
 * Copyright 2017 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.dialog;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.StockInfoBean;
import de.frankmuenster.jameica.depot.gui.controller.SearchDialogController;
import de.frankmuenster.jameica.depot.gui.parts.wpsearch.AbstractStockSearchPart;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * Suchfenster fr Wertpapiere
 *
 * @author Frank Mnster
 */
public class SearchDialog<I extends StockInfoBean, S extends AbstractStockSearchPart<I>>
		extends AbstractDialog<Object> {

	/**
	 * Der ausgewhlte Code
	 */
	private transient I bean;

	/**
	 * Der Suchtext
	 */
	// private final String searchText;

	/**
	 * Unser Kontrolleur
	 */
	private final transient SearchDialogController<I, S> controller;

	/**
	 * Liefert einen modalen Dialog, der mittel .open() angezeigt werden kann
	 *
	 * @param position
	 *            die Position, an der der Dialog angezeigt werden soll.
	 *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
	 * @param name
	 *            der Infobean aus einem ggf. vorbelegtem Feld
	 */
	public SearchDialog(final int position, final String name, final Class<S> searchPartClazz) {
		super(position, false);
		this.setTitle(Settings.getI18n().tr("SearchDialog.about")); //$NON-NLS-1$
		this.setPanelText(Settings.getI18n().tr("SearchDialog.panelText")); //$NON-NLS-1$
		this.controller = new SearchDialogController<I, S>(name, searchPartClazz);
		// this.searchText = name;
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected Object getData() {
		return this.bean;
	}

	/**
	 * @throws ApplicationException
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) throws RemoteException, ApplicationException {
		final ColumnLayout columns = new ColumnLayout(parent, 2);
		final Container links = new SimpleContainer(columns.getComposite());
		// rechte Seite
		final Container rechts = new SimpleContainer(columns.getComposite(), true, 1);
		final GridData gridData = (GridData) rechts.getComposite().getLayoutData();
		gridData.minimumWidth = 200;
		gridData.widthHint = 200;
		// final GridLayout glImage = new GridLayout();
		// glImage.numColumns = 2;
		// container.getComposite().setLayout(glImage);
		// final GridData gdImage = new GridData();
		// gdImage.grabExcessHorizontalSpace = true;
		// gdImage.horizontalAlignment = SWT.RIGHT;
		// gdImage.horizontalSpan = 2;
		Part part = this.controller.getSearchPart();
		part.paint(links.getComposite());
		// Felder
		links.addLabelPair(Settings.getI18n().tr("QuandlSearchDialog.codeLbl"), this.controller.getResultInput()); //$NON-NLS-1$
		// Beschreibung
		rechts.addHeadline(Settings.getI18n().tr("QuandlSearchDialog.describtionHeader")); //$NON-NLS-1$
		part = this.controller.getDescription();
		part.paint(rechts.getComposite());
		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(Settings.getI18n().tr("QuandlSearchDialog.create"), new Action() { //$NON-NLS-1$

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context) throws ApplicationException {
				SearchDialog.this.bean = SearchDialog.this.controller.getSelectedBean();
				SearchDialog.this.close();
			}
		}, null, true, "ok.png"); //$NON-NLS-1$
		buttons.addButton(Settings.getI18n().tr("QuandlSearchDialog.close"), new Action() { //$NON-NLS-1$

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
			 */
			@Override
			public void handleAction(final Object context) throws ApplicationException {
				SearchDialog.this.close();
			}
		}, null, true, "window-close.png"); //$NON-NLS-1$
		final SimpleContainer footer = new SimpleContainer(parent);
		footer.addButtonArea(buttons);
		this.setSize(650, SWT.DEFAULT);
	}
}
