/*
 * Copyright 2017 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.wpsearch;

import java.util.List;

import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooStockInfoBean;
import de.willuhn.jameica.gui.Part;

/**
 * Part das es erlaubt nach einem Wertpapier-Symbol zu suchen. Hier via YAHOO.
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class YahooSymbolSearchPart extends AbstractStockSearchPart<YahooStockInfoBean> implements Part {

	/**
	 * Das Composite in das wird das Suchpart malen.
	 */
	// private transient Composite inner;
	/**
	 * Standardkonstruktor mit Angabe eines Listeners
	 *
	 * @param listener
	 *            der zu benachrichtigende Listener
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public YahooSymbolSearchPart(final SearchResultSelectListener listener) {
		super(listener);
	}

	/**
	 * @see AbstractStockSearchPart#formatListEntry(de.frankmuenster.jameica.depot.finanzen.StockInfoBean)
	 */
	@Override
	protected String formatListEntry(final YahooStockInfoBean bean) {
		final StringBuffer result = new StringBuffer();
		StringBuffer help = new StringBuffer(bean.getSymbol()).append(AbstractStockSearchPart.SPACE_10);
		result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
		help = new StringBuffer(bean.getName()).append(AbstractStockSearchPart.SPACE_20);
		result.append(help.substring(0, 20)).append(AbstractStockSearchPart.SPACE_3);
		help = new StringBuffer(bean.getExchange()).append(AbstractStockSearchPart.SPACE_10);
		result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
		help = new StringBuffer(bean.getType()).append(AbstractStockSearchPart.SPACE_10);
		result.append(help.substring(0, 10)).append(AbstractStockSearchPart.SPACE_3);
		return result.toString();
	}

	/**
	 * @see AbstractStockSearchPart#perfromSearch(java.lang.String)
	 */
	@Override
	protected List<YahooStockInfoBean> perfromSearch(final String search) throws FinanceException {
		return YahooFinanceAPI.getInstance().lookUpQuote(search);
	}
}
