//
// Copyright 2017 frankmuenster.de
//
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
//   http://ec.europa.eu/idabc/eupl
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.jameica.depot.gui.views;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.controller.EditWertpapierController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 *
 */
public class EditWertpapierView extends AbstractView {

	/**
	 * Der Kontrolleur
	 */
	private transient EditWertpapierController control;

	/**
	 * Standardkonstruktor
	 */
	public EditWertpapierView() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@Override
	public void bind() throws Exception {
		GUI.getView().setTitle(Settings.getI18n().tr("EditWertpapierView.titel")); //$NON-NLS-1$

		this.control = new EditWertpapierController(this);

		// layout with 2 columns
		final ColumnLayout columns = new ColumnLayout(this.getParent(), 2);

		final Container links = new SimpleContainer(columns.getComposite());
		// Suchfenster

		// rechte Seite
		final Container right = new SimpleContainer(columns.getComposite(), true);
		GridData gridData = (GridData) right.getComposite().getLayoutData();
		gridData.minimumWidth = 250;
		right.addHeadline(Settings.getI18n().tr("EditWertpapierView.description")); //$NON-NLS-1$
		right.addInput(this.control.getComment());
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);
		right.addText(StringUtils.EMPTY, true);

		// Linke Seite
		// Anzeigen nur, wenn neues Wertpapier
		links.addHeadline(Settings.getI18n().tr("EditWertpapierView.searchYahoo")); //$NON-NLS-1$
		links.addPart(this.control.getYahooSearchPart());

		links.addHeadline(Settings.getI18n().tr("EditWertpapierView.yahooInfo")); //$NON-NLS-1$
		links.addLabelPair(Settings.getI18n().tr("EditWertpapierView.yahooSymbole"), this.control.getSymbol()); //$NON-NLS-1$
		links.addLabelPair(Settings.getI18n().tr("EditWertpapierView.yahooName"), this.control.getName()); //$NON-NLS-1$
		links.addText(StringUtils.EMPTY, true);

		// Whrung
		links.addHeadline(Settings.getI18n().tr("EditWertpapierView.currency")); //$NON-NLS-1$
		links.addLabelPair(Settings.getI18n().tr("EditWertpapierView.currency"), this.control.getStockCurrency()); //$NON-NLS-1$
		links.addText(StringUtils.EMPTY, true);

		// Historische Daten
		links.addHeadline(Settings.getI18n().tr("EditWertpapierView.historicalSearchParameter")); //$NON-NLS-1$
		final SimpleContainer container = new SimpleContainer(links.getComposite(), false, 3);
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = GridData.FILL;
		container.getComposite().setLayoutData(gridData);

		// FFM Instrument
		container.addLabelPair(Settings.getI18n().tr("EditWertpapierView.ffmInstrument"), //$NON-NLS-1$
				this.control.getFfmInstrument());
		this.control.getFfmSearchBtn().paint(container.getComposite());

		// Quandl Code
		container.addLabelPair(Settings.getI18n().tr("EditWertpapierView.quandlCode"), this.control.getQuandlCode()); //$NON-NLS-1$
		final Button quandlSearch = this.control.getQuandlSearchBtn();
		if (quandlSearch != null) {
			quandlSearch.paint(container.getComposite());
		}

		// Speicher und Abbrechen Buttons einfgen
		final ButtonArea buttons = new ButtonArea();
		final Button store = this.control.getStore();
		final Button cancel = this.control.getCancel();

		buttons.addButton(store);
		buttons.addButton(cancel);

		buttons.paint(this.getParent());

	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {
		super.unbind();
		this.control = null; // NOPMD
	}
}
