/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.action.EditWertpapierAction;
import de.frankmuenster.jameica.depot.gui.action.OrderDetailAction;
import de.frankmuenster.jameica.depot.gui.action.ShowAnalysen;
import de.frankmuenster.jameica.depot.gui.action.WertpapierAktualisiernAction;
import de.frankmuenster.jameica.depot.gui.action.WertpapierDeleteAction;
import de.frankmuenster.jameica.depot.gui.controller.WertpapierDetailController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.Headline;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 */
public class WertpapierDetailsView extends AbstractView {

	/**
	 * Der WertpapierDetailController
	 */
	private transient WertpapierDetailController control;

	/**
	 * Standardkonstruktor
	 */
	public WertpapierDetailsView() {
		super();
	}

	/**
	 * @throws RemoteException
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@SuppressWarnings("unused")
	@Override
	public void bind() throws RemoteException {
		// draw the title
		GUI.getView().setTitle(Settings.getI18n().tr("WertpapierDetailsView.viewTitle")); //$NON-NLS-1$
		// Instantiate controller
		this.control = new WertpapierDetailController(this);
		final Container container = new SimpleContainer(this.getParent());
		// layout with 2 columns
		final ColumnLayout columns = new ColumnLayout(container.getComposite(), 2);
		// left side
		this.leftContainer(columns.getComposite());
		// right side
		this.rightContainer(columns.getComposite());
		// zum Schluss die Grafik, falls es Daten gibt
		if (this.control.isHistoDataAvailable()) {
			new Headline(this.getParent(), Settings.getI18n().tr("WertpapierDetailsView.hlChart")); //$NON-NLS-1$
			this.control.getGrafik().paint(this.getParent());
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {
		super.unbind();
		this.control = null; // NOPMD
	}

	/**
	 * Linken Container malen.
	 *
	 * @param composite
	 * @throws RemoteException
	 */
	private void leftContainer(final Composite composite) throws RemoteException {
		final Container leftContainer = new SimpleContainer(composite);
		leftContainer.addHeadline(Settings.getI18n().tr("WertpapierDetailsView.hlOverview")); //$NON-NLS-1$
		leftContainer.addPart(this.control.getWertpapierDetails());
		final ButtonArea buttonArea = new ButtonArea();
		// Edit Button
		final EditWertpapierAction wpEditAction = new EditWertpapierAction();
		buttonArea.addButton(Settings.getI18n().tr("WertpapierDetailsView.editBtn"), wpEditAction, //$NON-NLS-1$
				this.control.getCurrentObject(), false, "document-open.png"); //$NON-NLS-1$
		// Refesh Button nur, wenn Papier auch in Yahoo
		if (((Wertpapier) this.control.getCurrentObject()).isInYahoo()) {
			final WertpapierAktualisiernAction wpUpdateAction = new WertpapierAktualisiernAction();
			wpUpdateAction.addListener(this.control);
			buttonArea.addButton(Settings.getI18n().tr("WertpapierDetailsView.btnRefresh"), //$NON-NLS-1$
					wpUpdateAction, this.control.getCurrentObject(), false, "view-refresh.png"); //$NON-NLS-1$
		} else {
			// Nur ne Leerzeile
			leftContainer.addText(StringUtils.EMPTY, true);
		}
		leftContainer.addButtonArea(buttonArea);
		// die Order History anzeigen
		leftContainer.addHeadline(Settings.getI18n().tr("WertpapierDetailsView.hlOrders")); //$NON-NLS-1$
		final SimpleContainer orderHistoryList = new SimpleContainer(leftContainer.getComposite(), false, 1);
		// Limit high to max 6 rows
		final GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.heightHint = 140; // height for ~6 rows
		orderHistoryList.getComposite().setLayoutData(gridData);
		// Tabelle muss mittels paint eingefgt werden, sonst taucht der
		// Scrollbalken rechts nicht auf
		final Part orderHistoryTbl = this.control.getOrderHistoryList();
		orderHistoryTbl.paint(orderHistoryList.getComposite());
		leftContainer.addText(StringUtils.EMPTY, true);
		// den Rest auch nur, wenn es Daten gibt
		if (this.control.isHistoDataAvailable()) {
			// und dann die Zeitachse
			leftContainer.addHeadline(Settings.getI18n().tr("WertpapierDetailsView.hlTimeRange")); //$NON-NLS-1$
			leftContainer.addPart(this.control.getZeitauswahl());
			leftContainer.addPart(this.control.getGrafikWerkzeuge());
		} else {
			leftContainer.addText(StringUtils.EMPTY, true);
			leftContainer.addText(StringUtils.EMPTY, true);
			leftContainer.addText(Settings.getI18n().tr("WertpapierDetailsView.noHistoricalData"), true, //$NON-NLS-1$
					Color.ERROR);
			leftContainer.addText(StringUtils.EMPTY, true);
			leftContainer.addText(StringUtils.EMPTY, true);
		}
	}

	/**
	 * Rechten Conatiner malen.
	 *
	 * @param composite
	 * @throws RemoteException
	 */
	private void rightContainer(final Composite composite) throws RemoteException {
		final Container rightContainer = new SimpleContainer(composite, true, 1);
		final GridData gridData = (GridData) rightContainer.getComposite().getLayoutData();
		gridData.minimumWidth = 280;
		rightContainer.getComposite().setLayoutData(gridData);
		rightContainer.addHeadline(Settings.getI18n().tr("WertpapierDetailsView.hlPerformance")); //$NON-NLS-1$
		rightContainer.addPart(this.control.getPerformanzPart());
		rightContainer.addHeadline(Settings.getI18n().tr("WertpapierDetailsView.hlComment")); //$NON-NLS-1$
		rightContainer.addInput(this.control.getComment());
		rightContainer.addText(StringUtils.EMPTY, true);
		rightContainer.addText(StringUtils.EMPTY, true);
		// add some buttons
		final ButtonArea buttonArea = new ButtonArea();
		// den Analyse Button nur, wenn es Daten gibt
		if (this.control.isHistoDataAvailable()) {
			buttonArea.addButton(Settings.getI18n().tr("WertpapierDetailsView.btnAnalysis"), //$NON-NLS-1$
					new ShowAnalysen(), this.control.getCurrentObject());
		}
		final Button saveWertpapier = this.control.getSave();
		buttonArea.addButton(saveWertpapier);
		buttonArea.addButton(Settings.getI18n().tr("WertpapierDetailsView.btnNewOrder"), //$NON-NLS-1$
				new OrderDetailAction(), this.control.getCurrentObject());
		buttonArea.addButton(Settings.getI18n().tr("WertpapierDetailsView.btnDelete"), //$NON-NLS-1$
				new WertpapierDeleteAction(), this.control.getCurrentObject());
		// Don't forget to paint the button area
		rightContainer.addButtonArea(buttonArea);
	}
}
